/*
 * SES GPIO functions
 *
 * Copyright 2006, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: wan_gpio.c,v 1.1 2007/02/14 17:23:52 dtutuianu Exp $
 */

#include <typedefs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <signal.h>
#include <assert.h>
#include <bcmnvram.h>
#include <bcmgpio.h>

#include "wan_gpio.h"

static int wan_led_gpio = BCMGPIO_UNDEFINED;

int wan_gpio_led_init()
{
	wan_led_gpio = 7;
	
	if (bcmgpio_connect(wan_led_gpio, BCMGPIO_DIRN_OUT) != 0) {
		printf("Error connecting GPIO %d to WAN led\n", wan_led_gpio);
		return -1;
	}

	return 0;
}

void wan_gpio_led_cleanup()
{
	if (wan_led_gpio != BCMGPIO_UNDEFINED) {
		bcmgpio_disconnect(wan_led_gpio);
		wan_led_gpio = BCMGPIO_UNDEFINED;
	}
}

void wan_led_on()
{
	unsigned long led_mask;
	
	printf("WAN LED on!\n");
	wan_gpio_led_init();
	
	if (wan_led_gpio == BCMGPIO_UNDEFINED)
		return;

	led_mask = ((unsigned long)1 << wan_led_gpio);
	bcmgpio_out(led_mask, led_mask);
	wan_gpio_led_cleanup();
}

void wan_led_off()
{
	unsigned long led_mask;
	unsigned long value;

	printf("WAN LED off!\n");
	wan_gpio_led_init();

	if (wan_led_gpio == BCMGPIO_UNDEFINED)
		return;

	led_mask = ((unsigned long)1 << wan_led_gpio);
	value = ((unsigned long) ~1 << wan_led_gpio);
	value &= led_mask;
	
	bcmgpio_out(led_mask, value);
	
	wan_gpio_led_cleanup();
}
